/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.server.DnsshimProtocolException;
import br.registro.dnsshim.domain.Zone;
import br.registro.dnsshim.domain.ZoneIncrement;
import br.registro.dnsshim.util.DomainNameUtil;
import br.registro.dnsshim.xfrd.domain.ZoneData;
import br.registro.dnsshim.xfrd.domain.ZoneInfo;
import br.registro.dnsshim.xfrd.domain.logic.ZoneInfoCache;
import br.registro.dnsshim.xfrd.repository.ZoneDataRepository;
import br.registro.dnsshim.xfrd.repository.ZoneIncrementRepository;
import br.registro.dnsshim.xfrd.repository.ZoneInfoRepository;
import br.registro.dnsshim.xfrd.repository.ZoneRepository;

public class ZoneDataDao implements ZoneDataRepository {
	private static final Logger logger = Logger.getLogger(ZoneDataDao.class);
	private ZoneRepository zoneRepository = new ZoneDao();
	private ZoneIncrementRepository zoneIncrementRepository = new ZoneIncrementDao(); 

	@Override
	public void save(ZoneData zoneData) throws IOException {
		String zonename = zoneData.getZonename();
		
		if (logger.isDebugEnabled()) {
			logger.debug("Saving ZoneData " + zonename);
		}
		
		Zone axfr = zoneData.getAxfr();
		zoneRepository.save(axfr);
				
		Collection<ZoneIncrement> ixfr = zoneData.getIncrements();
		for (ZoneIncrement increment : ixfr) {
			zoneIncrementRepository.save(increment);
		}	
		
		zoneIncrementRepository.verifySizeLimitIncrements(zonename);
	}

	public void delete(String zonename) throws IOException {
		zoneRepository.delete(zonename);
		zoneIncrementRepository.removeAll(zonename);
	}

	@Override
	public ZoneData findByZonename(String zonename)
		throws DnsshimProtocolException, IOException {
		String zn = DomainNameUtil.trimDot(zonename);
		if (logger.isDebugEnabled()) {
			logger.debug("Finding " + zn + " ZoneData");
		}
		
		ZoneInfoCache zoneInfoCache = ZoneInfoCache.getInstance();
		ZoneInfo zoneInfo = zoneInfoCache.get(zonename);
		if (zoneInfo == null) {
			return null;
		}
		
		Zone axfr = zoneRepository.findByZonename(zonename);
		if (axfr == null) {
			return null;
		}

		ZoneData zoneData = new ZoneData(zonename, zoneInfo.isSigned());
		Map<Long, ZoneIncrement> ixfrs = zoneIncrementRepository.findByZonename(zonename);

		zoneData.setAxfr(axfr);
		zoneData.setIxfrs(ixfrs);

		return zoneData;
	}


	@Override
	public Map<String, ZoneData> findAll()
		throws IOException, DnsshimProtocolException, ParseException {
		Map<String, ZoneData> zoneDatas = new LinkedHashMap<String, ZoneData>();
		ZoneInfoRepository zoneInfoRepository = new ZoneInfoDao();
		Map<String, ZoneInfo> all = zoneInfoRepository.findAll();
		for (Map.Entry<String, ZoneInfo> zi: all.entrySet()) {
			String zonename = zi.getValue().getZonename();
			
			Zone axfr = zoneRepository.findByZonename(zonename);
			Map<Long, ZoneIncrement> ixfrs =	zoneIncrementRepository.findByZonename(zonename);
			
			ZoneData zoneData = new ZoneData(zonename, zi.getValue().isSigned());
			zoneData.setAxfr(axfr);
			zoneData.setIxfrs(ixfrs);
			
			zoneDatas.put(zonename, zoneData);
		}
		return zoneDatas;
	}
}