/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.repository.FileSystemUtil;
import br.registro.dnsshim.xfrd.domain.Zonename;
import br.registro.dnsshim.xfrd.repository.ZonenameRepository;

public class ZonenameDao implements ZonenameRepository {
	private static final Logger logger = Logger.getLogger(ZonenameDao.class);
	private static final String EXTENSION = ".list";
	private static File file;

	public ZonenameDao() {
		StringBuilder root = new StringBuilder();
		try {
			// {DNSSHIM_HOME|user.home}/dnsshim/xfrd/conf/zonename.list
			if (System.getenv("DNSSHIM_HOME") == null) {
				root.append(System.getProperty("user.home"));
				root.append(File.separatorChar);
				root.append("dnsshim");
			} else {
				root.append(System.getenv("DNSSHIM_HOME"));
			}
			root.append(File.separatorChar);
			root.append("xfrd");
			root.append(File.separatorChar);
			root.append("conf");
			root.append(File.separatorChar);

			File directory = new File(root.toString());
			FileSystemUtil.makePath(directory);
			
			StringBuilder path = new StringBuilder(root);
			path.append("zonename");
			path.append(EXTENSION);
			
			file = new File(path.toString());
			
			if (!file.exists()) {
				if(!file.createNewFile()){
					throw new IOException("Cannot create file:"+file);
				}
			}
			
			if (logger.isDebugEnabled()) {
				logger.debug("Zonename base dir:"+root);
			}
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
	}

	@Override
	public List<Zonename> findAll() throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Loading zonenames...");
		}
		
		BufferedReader br = null; 
		List<Zonename> zonenames = new ArrayList<Zonename>();
		try {
			br = new BufferedReader(new FileReader(file));
			String line = null;
			while ((line = br.readLine()) != null) {
				if (!line.trim().isEmpty()) {
					zonenames.add(new Zonename(line));
				}	
			}
		} finally {
			br.close();
		}
		return zonenames;
	}

	@Override
	public void save(Zonename zonename) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving zonename...");
		}
		BufferedWriter bw = null;
		try {
			bw = new BufferedWriter(new FileWriter(file, true));
			bw.write(zonename.getName());
			bw.newLine();
		} finally {
			bw.close();
		}	
	}
	
	public void refresh(Set<String> zoneSet) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving zone list");
		}
		BufferedWriter bw = null;
		try {
			bw = new BufferedWriter(new FileWriter(file, false));
			for (String zone : zoneSet) {
				if (!zone.trim().isEmpty()) {
					bw.write(zone);
					bw.newLine();
				}	
			}
		} finally {
			bw.close();
		}
	}
	
}
