/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.domain.logic;

import java.io.IOException;
import java.util.List;

import org.apache.log4j.Logger;

import br.registro.dnsshim.domain.logic.CacheException;
import br.registro.dnsshim.domain.logic.CacheStrategy;
import br.registro.dnsshim.xfrd.dao.filesystem.SlaveGroupDao;
import br.registro.dnsshim.xfrd.domain.SlaveGroup;
import br.registro.dnsshim.xfrd.repository.SlaveGroupRepository;

public class SlaveGroupCache extends CacheStrategy<String, SlaveGroup> {
	private static final Logger logger = Logger.getLogger(SlaveGroupCache.class);
	private SlaveGroupRepository repository = new SlaveGroupDao();
	
	private static SlaveGroupCache instance;
	
	private SlaveGroupCache() {
		try {
			List<SlaveGroup> groups = repository.findAll();
			for (SlaveGroup slaveGroup : groups) {
				this.put(slaveGroup);
			}
		} catch (IOException e) {
			logger.error(e);
		}
	}

	@Override
	protected int getMaxEntriesInCache() {
		return 0;
	}

	@Override
	protected SlaveGroup restoreValue(String name) {
		try {
			return repository.findByName(name);
		} catch (IOException e) {
			logger.error(e);
			throw new CacheException(e.getMessage());
		}

	}

	@Override
	protected void storeValue(SlaveGroup slaveGroup) {
		try {
			repository.save(slaveGroup);
		} catch (IOException e) {
			logger.error(e);
			throw new CacheException(e.getMessage());
		}
	}
	
	public synchronized static SlaveGroupCache getInstance() {
		if (instance == null) {
			instance = new SlaveGroupCache();
		}
		return instance;
	}

}
